#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from elasticsearch_dsl import Q
from hypatia.interfaces import ICatalog
from hypatia.query import Any

from pyams_content.shared.view.interfaces import IWfView
from pyams_content.shared.view.interfaces.query import IViewUserQuery
from pyams_content_es.shared.view.interfaces import IEsViewUserQuery
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_utility


@adapter_config(name='audience',
                required=IWfView,
                provides=IViewUserQuery)
class CatalogAudienceQuery(ContextAdapter):
    """Catalog audience query"""

    @staticmethod
    def get_user_params(request):
        """User params getter"""
        audience = request.params.get('audience')
        if not audience:
            return
        if isinstance(audience, str):
            audience = audience.split(',')
        catalog = get_utility(ICatalog)
        yield Any(catalog['catalog_audience'], audience)


@adapter_config(name='audience',
                required=IWfView,
                provides=IEsViewUserQuery)
class EsCatalogAudienceQuery(ContextAdapter):
    """Elasticsearch catalog audience query"""

    @staticmethod
    def get_user_params(request):
        """User params getter"""
        audience = request.params.get('audience')
        if not audience:
            return
        if isinstance(audience, str):
            audience = audience.split(',')
        yield Q('terms',
                **{'catalog_info.audiences': audience})



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Or
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPForbidden
from zope.interface import Interface

from pyams_app_msc.feature.profile import IUserProfile
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import LOGIN_REFERER_KEY
from pyams_security.interfaces.plugin import IAuthenticatedPrincipalEvent
from pyams_security.utility import get_principal
from pyams_security_views.interfaces.login import ILoginView
from pyams_site.interfaces import ISiteRoot
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@adapter_config(required=(Interface, IPyAMSLayer, ILoginView),
                provides=IAJAXFormRenderer)
class LoginFormAJAXRenderer(ContextRequestViewAdapter):
    """Login form renderer"""

    def render(self, changes):  # pylint: disable=unused-argument
        """AJAX form renderer"""
        status = {
            'status': 'redirect'
        }
        request = self.request
        session = request.session
        hash = request.params.get('login_form.widgets.hash', '')
        if hash:
            if LOGIN_REFERER_KEY in session:
                status['location'] = f"{session[LOGIN_REFERER_KEY] or '/'}{hash}"
                del session[LOGIN_REFERER_KEY]
            else:
                status['location'] = f'/{hash}'
        else:
            principal_id = self.view.finished_state.get('changes')
            if principal_id:
                catalog = get_utility(ICatalog)
                query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                            Or(Eq(catalog['role:msc:manager'], principal_id),
                               Eq(catalog['role:msc:operator'], principal_id),
                               Eq(catalog['role:msc:contributor'], principal_id),
                               Eq(catalog['role:msc:reader'], principal_id)))
                theaters = list(CatalogResultSet(CatalogQuery(catalog).query(query)))
                if len(theaters) == 1:
                    theater = theaters[0]
                    status['location'] = absolute_url(theater, request, 'admin')
                elif len(theaters) > 1:
                    status['location'] = absolute_url(request.root, request,
                                                      'select-admin-theater.html')
                else:
                    session = request.session
                    if LOGIN_REFERER_KEY in session:
                        status['location'] = session[LOGIN_REFERER_KEY]
                        del session[LOGIN_REFERER_KEY]
                    else:
                        status['location'] = absolute_url(request.context, request)
        return status


@subscriber(IAuthenticatedPrincipalEvent)
def handle_authenticated_principal(event):
    """Check for inactive profile when authenticated"""
    principal = get_principal(principal_id=event.principal_id)
    profile = IUserProfile(principal)
    if not profile.active:
        raise HTTPForbidden(_("User profile is disabled!"))


@pagelet_config(name='select-admin-theater.html',
                context=ISiteRoot, layer=IPyAMSLayer)
@template_config(template='templates/select-admin-theater.pt',
                 layer=IPyAMSLayer)
class TheaterSelectionView(PortalContextIndexPage):
    """Theater selection view"""

    def get_theaters(self):
        """Theaters list getter"""
        principal_id = self.request.principal.id
        catalog = get_utility(ICatalog)
        query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                    Or(Eq(catalog['role:msc:manager'], principal_id),
                       Eq(catalog['role:msc:operator'], principal_id),
                       Eq(catalog['role:msc:contributor'], principal_id),
                       Eq(catalog['role:msc:reader'], principal_id)))
        yield from sorted(CatalogResultSet(CatalogQuery(catalog).query(query)),
                          key=lambda x: II18n(x).query_attribute('title', request=self.request))



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, timedelta

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, IBookingContainer, IBookingInfo, IBookingTarget
from pyams_app_msc.feature.booking.reminder import BookingReminderTask
from pyams_app_msc.feature.messaging.interfaces import IMessagingSettings
from pyams_app_msc.feature.planning.interfaces import IPlanning, ISession
from pyams_app_msc.feature.profile.interfaces import IUserProfile
from pyams_app_msc.feature.quotation import Quotation
from pyams_app_msc.interfaces import MANAGE_BOOKING_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPriceContainer
from pyams_catalog.utils import index_object
from pyams_content.feature.history.interfaces import IHistoryTarget
from pyams_content.interfaces import IObjectType
from pyams_file.interfaces.thumbnail import IThumbnails
from pyams_file.property import FileProperty
from pyams_i18n.interfaces import II18n
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_scheduler.interfaces import IScheduler
from pyams_scheduler.interfaces.task import IDateTaskScheduling, SCHEDULER_TASK_DATE_MODE
from pyams_security.interfaces import ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import FORBIDDEN_PERMISSION
from pyams_security.security import ProtectedObjectMixin
from pyams_security.utility import get_principal
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.date import format_date, format_datetime
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_pyramid_registry, get_utility, query_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_utils.zodb import load_object, volatile_property
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'


def get_booking_message_values(context, request, view):
    """Get booking message values"""
    theater = get_parent(context, IMovieTheater)
    # generate templates values
    session = ISession(context)
    entry = get_parent(session, ICatalogEntry)
    if entry is not None:
        versions = IWorkflowVersions(entry, None)
        if versions is not None:
            entry = versions.get_version(-1)
    values = {
        'theater_name': II18n(theater).query_attribute('title', request=request),
        'theater_email': theater.contact_email or '',
        'theater_phone': theater.phone_number or '',
        'theater_logo': '',
        'sender_name': request.principal.title,
        'sender_email': '',
        'booking_date': format_date(IZopeDublinCore(session).created),
        'session': get_object_label(session, request, view, name='short-text'),
        'session_title': get_object_label(entry, request, view),
        'session_date': format_datetime(session.start_date)
    }
    logo = theater.logo
    if logo:
        thumbnail = IThumbnails(logo).get_thumbnail('h160')
        values['theater_logo'] = (f'<img src="{absolute_url(thumbnail, request)}"'
                                  f' alt="Logo" />')
    mail_info = IPrincipalMailInfo(request.principal, None)
    if mail_info is not None:
        values['sender_email'] = next(mail_info.get_addresses())
    return values


@factory_config(IBookingInfo)
@implementer(IHistoryTarget)
class BookingInfo(ProtectedObjectMixin, Persistent, Contained):
    """Booking information"""

    creator = FieldProperty(IBookingInfo['creator'])
    recipient = FieldProperty(IBookingInfo['recipient'])
    status = FieldProperty(IBookingInfo['status'])
    nb_participants = FieldProperty(IBookingInfo['nb_participants'])
    nb_accompanists = FieldProperty(IBookingInfo['nb_accompanists'])
    nb_groups = FieldProperty(IBookingInfo['nb_groups'])
    price = FieldProperty(IBookingInfo['price'])
    accompanying_ratio = FieldProperty(IBookingInfo['accompanying_ratio'])
    cultural_pass = FieldProperty(IBookingInfo['cultural_pass'])
    comments = FieldProperty(IBookingInfo['comments'])
    notepad = FieldProperty(IBookingInfo['notepad'])
    archived = FieldProperty(IBookingInfo['archived'])
    quotation_number = FieldProperty(IBookingInfo['quotation_number'])
    quotation_message = FieldProperty(IBookingInfo['quotation_message'])
    quotation = FileProperty(IBookingInfo['quotation'])
    send_update = FieldProperty(IBookingInfo['send_update'])
    update_subject = FieldProperty(IBookingInfo['update_subject'])
    update_message = FieldProperty(IBookingInfo['update_message'])
    send_reminder = FieldProperty(IBookingInfo['send_reminder'])
    reminder_subject = FieldProperty(IBookingInfo['reminder_subject'])
    reminder_message = FieldProperty(IBookingInfo['reminder_message'])
    reminder_date = FieldProperty(IBookingInfo['reminder_date'])

    def get_recipient(self, with_info=True):
        """Recipient principal getter"""
        principal = get_principal(None, principal_id=self.recipient)
        if with_info:
            profile = IUserProfile(principal, None)
            return principal, profile
        return principal

    @property
    def nb_seats(self):
        """Total seats count"""
        return (self.nb_participants or 0) + (self.nb_accompanists or 0)

    def get_price(self):
        """Price getter"""
        if not self.price:
            return None
        theater = IMovieTheater(self)
        return ICinemaPriceContainer(theater).get(self.price)

    @volatile_property
    def session(self):
        """Booking session getter"""
        return get_parent(self, IBookingTarget)

    @property
    def session_index(self):
        """Booking session index value getter"""
        return ICacheKeyValue(self.session)

    def set_session(self, new_session):
        """Update booking session"""
        if not ISession.providedBy(new_session):
            new_session = load_object(new_session, self)
        if (new_session is self.session) or not ISession.providedBy(new_session):
            return
        old_bookings = self.__parent__
        new_bookings = IBookingContainer(new_session)
        new_bookings.append(self)
        index_object(self)
        del self.session
        if self.quotation is not None:
            del self.quotation
            self.get_quotation(force_refresh=True)
        del old_bookings[ICacheKeyValue(self)]
        registry = get_pyramid_registry()
        registry.notify(ObjectModifiedEvent(self))
        return self

    def get_quotation_number(self):
        theater = IMovieTheater(self)
        settings = IMovieTheaterSettings(theater)
        return settings.get_quotation_number()

    def get_quotation(self, force_refresh=False, store=True, **kwargs):
        quotation = self.quotation
        if force_refresh or (not quotation):
            quotation_number = self.quotation_number
            if not quotation_number:
                quotation_number = self.quotation_number = self.get_quotation_number()
            quotation = (f'{quotation_number}.pdf',
                         bytes(Quotation(self, quotation_number, **kwargs)))
            if store:
                self.quotation = quotation
        return quotation

    def send_reminder_message(self):
        """Send reminder message"""
        if not self.send_reminder:
            return
        root = get_parent(self, ISiteRoot)
        settings = IMessagingSettings(root, None)
        if settings is None:
            return
        mailer = settings.get_mailer()
        if mailer is None:
            return
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(self.recipient)
        mail_info = IPrincipalMailInfo(principal, None)
        if mail_info is None:
            return
        mail_addresses = [
            f'{name} <{address}>'
            for name, address in mail_info.get_addresses()
        ]
        message = HTMLMessage(f'{settings.subject_prefix} {self.reminder_subject}',
                              from_addr=f'{settings.source_name} <{settings.source_address}>',
                              to_addr=mail_addresses,
                              html=self.reminder_message)
        mailer.send(message)
        self.reminder_date = tztime(datetime.utcnow())


@adapter_config(required=IBookingInfo,
                provides=IObjectType)
def booking_object_type(context):
    """Booking object type value adapter"""
    return get_interface_base_name(IBookingInfo)


@adapter_config(required=IBookingInfo,
                provides=IViewContextPermissionChecker)
class BookingInfoPermissionChecker(ContextAdapter):
    """Booking permission checker"""

    @property
    def edit_permission(self):
        if self.context.archived:
            return FORBIDDEN_PERMISSION
        return MANAGE_BOOKING_PERMISSION


@adapter_config(name='delete',
                required=IBookingInfo,
                provides=IViewContextPermissionChecker)
class BookingInfoDeletePermissionChecker(ContextAdapter):
    """Booking delete permission checker"""

    @property
    def edit_permission(self):
        if self.context.archived or (self.context.status == BOOKING_STATUS.ACCEPTED.value):
            return FORBIDDEN_PERMISSION
        return MANAGE_BOOKING_PERMISSION


@adapter_config(required=IBookingInfo,
                provides=IBookingContainer)
def booking_container_adapter(context):
    """Booking container adapter"""
    return context.__parent__


@adapter_config(required=IBookingInfo,
                provides=ISession)
def booking_session_adapter(context):
    """Booking session adapter"""
    return context.session


@adapter_config(required=IBookingInfo,
                provides=IPlanning)
def booking_planning_adapter(context):
    """Booking planning adapter"""
    return get_parent(context, IPlanning)


@adapter_config(required=IBookingInfo,
                provides=IMovieTheater)
def booking_theater_adapter(context):
    """Booking theater adapter"""
    return get_parent(context, IMovieTheater)


@subscriber(IObjectModifiedEvent, context_selector=IBookingInfo)
def handle_modified_booking(event):
    """Handle modified booking"""
    scheduler = query_utility(IScheduler)
    if scheduler is None:
        return
    booking = IBookingInfo(event.object)
    theater = IMovieTheater(booking)
    task_id = f'reminder::{ICacheKeyValue(booking)}'
    if task_id in scheduler:
        del scheduler[task_id]
    settings = IMovieTheaterSettings(theater)
    if not settings.reminder_delay:
        return
    reminder_date = booking.session.start_date - timedelta(days=settings.reminder_delay)
    if reminder_date < tztime(datetime.utcnow()):
        return
    if booking.send_reminder and not booking.reminder_date:
        task = BookingReminderTask(booking)
        task.name = f'Booking {task_id}'
        task.schedule_mode = SCHEDULER_TASK_DATE_MODE
        scheduler_info = IDateTaskScheduling(task)
        scheduler_info.start_date = reminder_date
        scheduler_info.active = True
        scheduler[task_id] = task



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.profile.skin module

"""

from zope.interface import Interface

from pyams_app_msc.feature.profile import IUserProfile
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.form import EditForm
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import LOGIN_REFERER_KEY
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@ajax_form_config(name='my-profile.html',
                  layer=IPyAMSLayer)
class UserProfileEditForm(EditForm, PortalContextIndexPage):
    """User profile edit form"""

    title = _("My user profile")
    legend = _("Account settings")

    fields = Fields(IUserProfile).omit('active')

    def update_actions(self):
        super().update_actions()
        action = self.actions.get('apply')
        if action is not None:
            action.add_class('btn-primary')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        email = self.widgets.get('email')
        if email is not None:
            email.readonly = 'readonly'


@adapter_config(required=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IFormContent)
def user_profile_edit_form_content(context, request, form):
    """User profile edit form content getter"""
    return IUserProfile(request)


@adapter_config(required=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IAJAXFormRenderer)
class UserProfileEditFormRenderer(ContextRequestViewAdapter):
    """User profile edit form renderer"""

    def render(self, changes):
        result = {
            'status': 'redirect'
        }
        session = self.request.session
        if LOGIN_REFERER_KEY in session:
            result['location'] = f"{session[LOGIN_REFERER_KEY] or '/'}"
            del session[LOGIN_REFERER_KEY]
        else:
            result['location'] = '/'
        return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, timedelta

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.booking.interfaces import IBookingContainer
from pyams_app_msc.feature.planning.interfaces import ISession, VERSION_INFO, VERSION_INFO_ABBR
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntry
from pyams_app_msc.shared.catalog.portlet.skin.interfaces import ICatalogViewItemsPortletPanelsRendererSettings
from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.view.portlet.interfaces import IViewItemsPortletSettings
from pyams_content.shared.view.portlet.skin import ViewItemsPortletPanelsRenderer, \
    ViewItemsPortletPanelsRendererSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(ICatalogViewItemsPortletPanelsRendererSettings)
class CatalogViewItemsPortletPanelsRendererSettings(ViewItemsPortletPanelsRendererSettings):
    """Catalog view items portlet panels renderer settings"""

    display_sessions = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['display_sessions'])
    sessions_weeks = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['sessions_weeks'])
    display_free_seats = FieldProperty(ICatalogViewItemsPortletPanelsRendererSettings['display_free_seats'])


@adapter_config(name='catalog-panels',
                required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-catalog-panels.pt',
                 layer=IPyAMSLayer)
class CatalogViewItemsPortletPanelsRenderer(ViewItemsPortletPanelsRenderer):
    """Catalog view items portlet catalog panels renderer"""

    label = _("Catalog entries with next planned sessions")
    weight = 40

    settings_interface = ICatalogViewItemsPortletPanelsRendererSettings

    def get_sessions(self, wf_entry):
        """Display incoming sessions for provided catalog entry"""
        entry = get_parent(wf_entry, ICatalogEntry)
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        now = tztime(datetime.utcnow())
        query = And(Eq(catalog['object_types'], get_interface_base_name(ISession)),
                    Eq(catalog['parents'], intids.queryId(entry)),
                    Ge(catalog['planning_start_date'], now),
                    Le(catalog['planning_end_date'], now + timedelta(weeks=self.renderer_settings.sessions_weeks)))
        yield from sorted(filter(lambda x: x.extern_bookable,
                                 CatalogResultSet(CatalogQuery(catalog).query(query))),
                          key=lambda x: x.start_date)

    def get_version(self, session):
        """Get displayed version of provided entry"""
        if not session.version:
            return ''
        version = VERSION_INFO_ABBR.get(VERSION_INFO(session.version))
        if not version:
            return ''
        translate = self.request.localizer.translate
        return f'- {translate(version)}'

    @staticmethod
    def get_free_seats(session):
        """Get free seats for provided session"""
        bookings = IBookingContainer(session, None)
        if bookings is None:
            return session.capacity, session.capacity
        return session.capacity - bookings.get_confirmed_seats(), session.capacity



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.decorator import reify
from zope.interface import Interface
from zope.schema import Bool, Text, TextLine

from pyams_app_msc.feature.booking import IBookingContainer, IBookingInfo
from pyams_app_msc.feature.planning import IPlanning, IPlanningTarget
from pyams_app_msc.feature.planning.interfaces import VERSION_INFO_VOCABULARY
from pyams_app_msc.interfaces import CREATE_BOOKING_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.schema.button import SubmitButton
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import Viewlet, viewlet_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class IBookingAddFormInfo(IBookingInfo):
    """Booking add form interface"""

    session_id = TextLine(title=_("Session ID"),
                          required=True)

    comments = Text(title=_("Comments"),
                    description=_("You can add optional comments to your booking request"),
                    required=False)

    send_confirmation = Bool(title=_("Get confirmation message?"),
                             description=_("If 'yes', a confirmation message will be sent to you to "
                                           "acknowledge the booking"),
                             required=True,
                             default=True)


class IBookingAddFormButtons(Interface):
    """Booking add form buttons interface"""

    add = SubmitButton(name='add',
                       title=_("Add booking for this session"))


@ajax_form_config(name='booking-new.html',
                  context=IPlanningTarget, layer=IPyAMSLayer,
                  permission=CREATE_BOOKING_PERMISSION)
class BookingAddForm(AddForm, PortalContextIndexPage):
    """Booking add form"""

    legend = _("Your new booking properties")

    fields = Fields(IBookingAddFormInfo).select('session_id', 'nb_participants',
                                                'nb_accompanists', 'nb_groups', 'cultural_pass',
                                                'comments', 'send_confirmation')
    buttons = Buttons(IBookingAddFormButtons)

    content_factory = IBookingInfo
    _edit_permission = CREATE_BOOKING_PERMISSION

    @reify
    def session(self):
        """Session getter"""
        session_id_widget = self.widgets.get('session_id')
        if session_id_widget is not None:
            session_id = session_id_widget.value
        else:
            session_id = self.request.params.get('session_id')
        if session_id:
            planning = IPlanning(self.context)
            return planning.get(session_id)
        return None

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        session_id = self.widgets.get('session_id')
        if session_id is not None:
            session_id.mode = HIDDEN_MODE
            if 'session_id' in self.request.params:
                session_id.value = self.request.params['session_id']

    @handler(buttons['add'])
    def handle_add(self, action):
        super().handle_add(self, action)

    def update_content(self, obj, data):
        data = data.get(self, data)
        for name in ('nb_participants', 'nb_accompanists', 'nb_groups', 'cultural_pass',
                     'comments', 'send_confirmation'):
            setattr(obj, name, data.get(name))
        obj.recipient = self.request.principal.id
        obj.creator = self.request.principal.id

    def add(self, obj):
        del obj.session
        container = IBookingContainer(self.session, None)
        if container is not None:
            container.append(obj)


@viewlet_config(name='booking-new.header',
                context=IPlanningTarget, layer=IPyAMSLayer, view=BookingAddForm,
                manager=IFormHeaderViewletManager, weight=10)
@template_config(template='templates/booking-new-header.pt',
                 layer=IPyAMSLayer)
class NewBookingHeader(Viewlet):
    """New booking header viewlet"""

    @property
    def start_date(self):
        start_date = self.view.session.start_date
        return format_datetime(start_date)

    def get_version(self):
        version = VERSION_INFO_VOCABULARY.by_value.get(self.view.session.version)
        if not version:
            return None
        return self.request.localizer.translate(version.title)


@adapter_config(required=(IPlanningTarget, IPyAMSLayer, BookingAddForm),
                provides=IAJAXFormRenderer)
class BookingAddFormRenderer(ContextRequestViewAdapter):
    """Booking add form renderer"""

    def render(self, changes):
        if changes is None:
            return
        return {
            'status': 'redirect',
            'location': absolute_url(self.context, self.request, 'booking-ok.html')
        }


@pagelet_config(name='booking-ok.html',
                layer=IPyAMSLayer)
@template_config(template='templates/booking-ok.pt',
                 layer=IPyAMSLayer)
class BookingOKView(PortalContextIndexPage):
    """Booking acknowledge view"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from urllib.parse import urlencode

from zope.interface import Interface

from pyams_app_msc.feature.planning.interfaces import VERSION_INFO, VERSION_INFO_LABEL
from pyams_app_msc.shared.catalog import ICatalogEntryInfo, IWfCatalogEntry
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.shared.common.portlet.interfaces import ISpecificitiesRenderer
from pyams_content.skin.interfaces import IPublicURL
from pyams_i18n.language import BASE_LANGUAGES
from pyams_layer.interfaces import IPyAMSLayer
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces.url import ICanonicalURL
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import ViewContentProvider

from pyams_app_msc import _


@adapter_config(required=(IWfCatalogEntry, IPyAMSLayer, Interface),
                provides=ISpecificitiesRenderer)
@template_config(template='templates/specificities.pt', layer=IPyAMSLayer)
class CatalogEntrySpecificitiesRenderer(ViewContentProvider):
    """Catalog entry specificities renderer"""

    entry_info = None

    def update(self):
        super().update()
        self.entry_info = ICatalogEntryInfo(self.context, None)

    def render(self, template_name=''):
        if self.entry_info is None:
            return ''
        return super().render(template_name)

    def get_language(self, value):
        translate = self.request.localizer.translate
        return translate(BASE_LANGUAGES.get(value, _("(unknown)")))

    def get_version(self, value):
        translate = self.request.localizer.translate
        return translate(VERSION_INFO_LABEL.get(VERSION_INFO(value), _("(undefined)")))

    def get_duration(self, value):
        translate = self.request.localizer.translate
        hours = value // 60
        minutes = value % 60
        if hours == 0:
            return translate(_("{} minutes")).format(minutes)
        return translate(_("{}h {}min")).format(hours, minutes)


@adapter_config(required=(IWfCatalogEntry, IPyAMSLayer),
                provides=ICanonicalURL)
class CatalogEntryCanonicalURL(ContextRequestAdapter):
    """Catalog entry canonical URL"""

    def get_url(self, view_name=None, query=None):
        theater = get_parent(self.context, IMovieTheater)
        query = urlencode(query) if query else None
        return absolute_url(theater, self.request,
                            f"+/{ISequentialIdInfo(self.context).get_base_oid().strip()}"
                            f"::{self.context.content_url}"
                            f"{'/{}'.format(view_name) if view_name else '.html'}"
                            f"{'?{}'.format(query) if query else ''}")


@adapter_config(name='booking-new.html',
                required=(IWfCatalogEntry, IPyAMSLayer),
                provides=IPublicURL)
class CatalogEntryPublicURL(ContextRequestAdapter):
    """Catalog entry public URL"""

    def get_url(self):
        """Public URL getter"""
        return self.request.url
